import cupy as cp  # type: ignore

class CrossA0001:
    def __init__(self) -> None: ...
    def _crossA0001(
        self,
        X: cp.core.core.ndarray,
        digits: int,
        repetitions: int,
        n_samples: int,
        percent_c: float,
    ) -> cp.core.core.ndarray: ...
