import cupy as cp  # type: ignore

class PermutationA0001:
    def __init__(self) -> None: ...
    def _permutationA0001(
        self, digits: int, repetitions: int, n_samples: int
    ) -> cp.core.core.ndarray: ...
